class Review < ActiveRecord::Base
  belongs_to :reviewable, polymorphic: true
  belongs_to :user

  has_many :review_ratings, dependent: :destroy
  has_many :ratings, through: :review_ratings
  has_many :rates, dependent: :destroy

  validates_presence_of :user, :reviewable
  validates :reviewable_type, inclusion: {in: %w(LearningObject Collection), message: '%{value} is not a valid reviewable type!'}

  default_scope { includes(:user) }

  def rating_values
    review_ratings.includes(:rating).map { |r| {rating: r.rating, value: r.value} }
  end

  def rating_average
    review_ratings.average(:value).floor(1).to_f
  end

  def rates_percentage
    return false if rates.count < 5
    approved = rates.where(approves: true).count
    (approved.to_f / rates.count.to_f) * 100
  end

end
