class ScoreCalculatorService

  def initialize(object)
    @object = object
    @class = @object.class.name
  end

  def calculate
    case @class
    when 'User' then reputation
    end
  end

  def reputation(values = nil)
    values = reputation_hash if values.nil?

    values = values.with_indifferent_access

    sum = 0

    Score.all.find_each do |score|
      max = score.max_value.to_f

      next if max == 0

      sum += (values[score.code].to_f / max) * score.weight.to_f
    end

    (sum.to_f / sum_weights.to_f).round(3)
  end

  private

  def reputation_hash
    {
      'reputation_submitted': @object.learning_objects.count,
      'reputation_reviews_average_score': @object.learning_objects.review_ratings_average,
      'reputation_added_in_best_collections': @object.learning_objects_in_best_collections,
      'reputation_best_score': @object.learning_objects.maximum(:score),
      'reputation_average_score': @object.learning_objects.average(:score),
      'reputation_reviews_rate': @object.review_approval_average,
      # 'reputation_confirmed_complaints': ,
      'reputation_followers': @object.followers.count,
      'reputation_submitted_recently': @object.learning_objects_submitted_recently,
      'reputation_collection_score': @object.where(privacy: 'public').average(:score)
    }
  end

  def sum_weights
    Score.pluck(:weight).inject(0.0) { |a, e| a + e }
  end

end
