module Thumbnail
  module Generate

    include RepositoriesProxy
    # include Bitstream::Utils
    # include Bitstream::AcceptedFormats

    def generate_thumbnail(learning_object_id)

      size = "530x298"

      @item = learning_object_repository.find learning_object_id
      return false if @item.nil?

      puts "#{method_log_tag} Starting to generate thumbnail"

      item_uniq_hash = encode_hash_from @item.id.to_s + @item.name

      workdir = tmpdir_path(item_uniq_hash)
      thumbnail = thumbnail_path(item_uniq_hash, size)
      output = "#{public_dir}#{thumbnail}"

      accepted_files = get_accepted_bitstreams(@item.get_filename,@item.retrievelink,workdir)
      accepted_files.sort! { |x,y| File.size(y) <=> File.size(x) }
      accepted_files.each do |input|
        filename = get_file_basename input
        file_format = get_file_extname input
        puts "#{method_log_tag} Trying to generate thumbnail for '#{filename}'"
        begin
          if accepted_video_formats.include? file_format
            generate_video_thumbnail(input, output, size)
          else
            generate_image_thumbnail(input, output, size)
          end
        rescue Exception => e
          puts "#{method_log_tag} #{e}
              \r#{method_log_tag} ERROR: input='#{input}'
              \r#{method_log_tag} RESCUE: Try the next input.."
        else
          learning_object_repository.update_property(@item,"thumbnail",thumbnail)

          clean_tmpfiles(workdir, accepted_files)
          puts "#{method_log_tag} SUCCESS: input='#{input}'"
          return true
        end
      end

      clean_tmpfiles(workdir, accepted_files)
      puts "#{method_log_tag} ERROR: End of input files!!!"
      return false

    end

    def default_thumbnail
      @default_thumbnail ||= ''
    end

    private

    def generate_video_thumbnail(input,output,size)
      movie = FFMPEG::Movie.new(input)
      if movie.valid? and !movie.video_codec.nil?
        frame = (movie.duration * 25/100).floor
        loop do
          begin
            movie.screenshot(output,
                    { seek_time: frame, resolution: size },
                    preserve_aspect_ratio: :width)
          rescue
            if frame > 0
              puts "#{method_log_tag} Decreasing frame and trying again!!! "
              frame = (frame * 0.50).floor
            else
              raise "#{method_log_tag} ERROR: Video's thumbnail was not generated. (FRAME_ERROR)"
            end
          else
            return true, "#{method_log_tag} Video's thumbnail was successfully generated!"
          end
        end
      else
        raise "#{method_log_tag} ERROR: Video's thumbnail was not generated. (NULL_CODEC)"
      end
    end

    def generate_image_thumbnail(input,output,size)
      # Read the image and resize it. The `change_geometry' method
      # computes the new image geometry and yields to a block. The
      # return value of the block is the return value of the method.
      img = Magick::Image.read(input)[0]
      img.alpha(Magick::DeactivateAlphaChannel)
      img.change_geometry!(size) { |cols, rows| img.thumbnail! cols, rows }
      img.write(output)
    end

    def encode_hash_from(object)
      Digest::SHA1.hexdigest object
    end

    def thumbnail_path(id, size)
      thumbnails_dir = "/thumbnails"
      create_dir "#{public_dir}#{thumbnails_dir}"
      return "#{thumbnails_dir}/#{id}_#{size}.jpg"
    end

    def item_id
      @item_id ||= @item.id
    end

    def method_log_tag
      return "[#{caller_locations(1,1)[0].label} (#{item_id})]"
    end

  end
end
